@@ -38,7 +38,7 @@ static struct io_u *fio_skeleton_event(struct thread_data *td, int event)
  * numbers. Required.
  */
 static int fio_skeleton_getevents(struct thread_data *td, unsigned int min,
-				  unsigned int max, struct timespec *t)
+				  unsigned int max, const struct timespec *t)
 {
 	return 0;
 }
@@ -99,7 +99,7 @@ static int fio_skeleton_init(struct thread_data *td)
 }
 
 /*
- * This is paired with the ->init() funtion and is called when a thread is
+ * This is paired with the ->init() function and is called when a thread is
  * done doing io. Should tear down anything setup by the ->init() function.
  * Not required.
  */
